/**
 *  @file getpid.c
 *
 *  @copyright Copyright (c) 2015 Ambarella, Inc.
 *
 *  This file and its contents ("Software") are protected by intellectual property rights including, without limitation,
 *  U.S. and/or foreign copyrights.  This Software is also the confidential and proprietary information of Ambarella, Inc.
 *  and its licensors.  You may not use, reproduce, disclose, distribute, modify, or otherwise prepare derivative works
 *  of this Software or any portion thereof except pursuant to a signed license agreement or nondisclosure agreement with
 *  Ambarella, Inc. or its authorized affiliates.  In the absence of such an agreement, you agree to promptly notify and
 *  return this Software to Ambarella, Inc.
 *
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *  WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  AMBARELLA, INC. OR ITS AFFILIATES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  COMPUTER FAILURE OR MALFUNCTION; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  @details _getpid()
 *
 */

#include <sc.h>

#ifdef CONFIG_SSP_THREADX_SMP
#include <ssp/threadx/tx_api_SMP.h>
#else
#include <ssp/threadx/tx_api_Uniprocessor.h>
#endif

#ifdef CONFIG_SSP_THREADX_POSIX
#include <pthread.h>

extern INT is_posix_thread(TX_THREAD *thread_ptr);
#endif

#include <ssp/AmbaSSP.h>
#include <AmbaPrint.h>
#include <ssp/soc/intrinsics.h>

/* Should always be successful.
 * return pthreadID if have POSIX wrapper
 * otherwise, return TX_THREAD
 */
int _getpid(void)
{
    int pid;

    pid = (int)tx_thread_identify();

    AmbaPrint("%s, caller pid: %d(0x%X)", __func__, pid, pid);
    AmbaPrint("TODO: %s()", __func__);
    K_ASSERT(0);

    if (pid == 0) {
        AmbaPrint("%s: Invalid PID 0.", __func__);
        AmbaKAL_TaskSleep(10);
        return 0;
    }

#ifdef CONFIG_SSP_THREADX_POSIX
    if (is_posix_thread((TX_THREAD *)pid)) {
        /* Thread was created by POSIX wrapper. */
        return ((POSIX_TCB *)pid)->pthreadID;
    }
#endif

    return pid;
}

